@@ -14,13 +14,11 @@
 #include "client.h"
 #include "socket.h"
 
-#define MAX_FDS 1024 * 8
 #define ACCEPT_TIMEOUT_SECS 1
 #define SHORT_TIMEOUT_SECS 2
 #define WRITE_TIMEOUT_SECS 60 * 5 
-#define READ_LONG_TIMEOUT_SECS 5
+//#define READ_LONG_TIMEOUT_SECS 3
 
-#define BACKLOG 1024 * 8
 #define MAX_BUFSIZE 1024 * 8
 #define INPUT_BUF_SIZE 1024 * 8
 
@@ -42,12 +40,16 @@ static char *error_log_path = NULL; //error log path
 static int err_log_fd = -1; //error log
 
 static int is_keep_alive = 0; //keep alive support
-//static int keep_alive_timeout = 5;
+static int keep_alive_timeout = 5;
 
 int max_content_length = 1024 * 1024 * 16; //max_content_length
 
 static char *unix_sock_name = NULL;
 
+static int backlog = 1024 * 4; // backlog size
+static int max_fd = 1024 * 4;  // picoev max_fd
+
+// greenlet hub switch value
 PyObject *hub_switch_value;
 
 
@@ -138,7 +140,7 @@ close_conn(client_t *cli, picoev_loop* loop)
         new_client = new_client_t(cli->fd, cli->remote_addr, cli->remote_port);
         new_client->keep_alive = 1;
         init_parser(new_client, server_name, server_port);
-        picoev_add(main_loop, new_client->fd, PICOEV_READ, READ_LONG_TIMEOUT_SECS, r_callback, (void *)new_client);
+        picoev_add(main_loop, new_client->fd, PICOEV_READ, keep_alive_timeout, r_callback, (void *)new_client);
     }
     PyMem_Free(cli);
 
@@ -470,15 +472,24 @@ r_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
                 if (errno == EAGAIN || errno == EWOULDBLOCK) { /* try again later */
                     break;
                 } else { /* fatal error */
-                    PyErr_SetFromErrno(PyExc_IOError);
-                    write_error_log(__FILE__, __LINE__); 
-                    cli->keep_alive = 0;
-                    cli->status_code = 500;
-                    if(errno != ECONNRESET){
-                        send_error_page(cli);
-                    }else{
+                    if(cli->keep_alive && errno == ECONNRESET){
+                        
+                        cli->keep_alive = 0;
+                        cli->status_code = 500;
                         cli->header_done = 1;
                         cli->response_closed = 1;
+                        
+                    }else{
+                        PyErr_SetFromErrno(PyExc_IOError);
+                        write_error_log(__FILE__, __LINE__); 
+                        cli->keep_alive = 0;
+                        cli->status_code = 500;
+                        if(errno != ECONNRESET){
+                            send_error_page(cli);
+                        }else{
+                            cli->header_done = 1;
+                            cli->response_closed = 1;
+                        }
                     }
                     close_conn(cli, loop);
                     return;
@@ -566,7 +577,7 @@ accept_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
             remote_port = ntohs(client_addr.sin_port);
             client = new_client_t(client_fd, remote_addr, remote_port);
             init_parser(client, server_name, server_port);
-            picoev_add(loop, client_fd, PICOEV_READ, READ_LONG_TIMEOUT_SECS, r_callback, (void *)client);
+            picoev_add(loop, client_fd, PICOEV_READ, keep_alive_timeout, r_callback, (void *)client);
         }else{
             if (errno != EAGAIN && errno != EWOULDBLOCK) {
                 PyErr_SetFromErrno(PyExc_IOError);
@@ -646,8 +657,8 @@ inet_listen(void)
 
     freeaddrinfo(servinfo); // all done with this structure
     
-    // BACKLOG 1024
-    if (listen(listen_sock, BACKLOG) == -1) {
+    // BACKLOG 
+    if (listen(listen_sock, backlog) == -1) {
         close(listen_sock);
         PyErr_SetFromErrno(PyExc_IOError);
         return -1;
@@ -705,7 +716,7 @@ unix_listen(char *sock_name)
     umask(old_umask);
 
     // BACKLOG 1024
-    if (listen(listen_sock, BACKLOG) == -1) {
+    if (listen(listen_sock, backlog) == -1) {
         close(listen_sock);
         PyErr_SetFromErrno(PyExc_IOError);
         return -1;
@@ -835,7 +846,7 @@ meinheld_run_loop(PyObject *self, PyObject *args)
     setup_server_env();
 
     /* init picoev */
-    picoev_init(MAX_FDS);
+    picoev_init(max_fd);
     /* create loop */
     main_loop = picoev_create_loop(60);
     loop_done = 1;
@@ -861,7 +872,7 @@ meinheld_run_loop(PyObject *self, PyObject *args)
             i = 0;
         }
 #ifdef DEBUG
-        printf("loop \n");
+        //printf("loop \n");
 #endif
     }
 
@@ -892,10 +903,15 @@ meinheld_set_keepalive(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "i", &on))
         return NULL;
     if(on < 0){
-        PyErr_SetString(PyExc_ValueError, "must be 0 or 1");
+        PyErr_SetString(PyExc_ValueError, "keep alive value out of range ");
         return NULL;
     }
     is_keep_alive = on;
+    if(is_keep_alive){
+        keep_alive_timeout = on;
+    }else{
+        keep_alive_timeout = 2;
+    }
     Py_RETURN_NONE;
 }
 
@@ -905,12 +921,57 @@ meinheld_get_keepalive(PyObject *self, PyObject *args)
     return Py_BuildValue("i", is_keep_alive);
 }
 
+PyObject *
+meinheld_set_backlog(PyObject *self, PyObject *args)
+{
+    int temp;
+    if (!PyArg_ParseTuple(args, "i", &temp))
+        return NULL;
+    if(temp <= 0){
+        PyErr_SetString(PyExc_ValueError, "backlog value out of range ");
+        return NULL;
+    }
+    backlog = temp;
+    Py_RETURN_NONE;
+}
+
+PyObject *
+meinheld_get_backlog(PyObject *self, PyObject *args)
+{
+    return Py_BuildValue("i", backlog);
+}
+
+PyObject *
+meinheld_set_picoev_max_fd(PyObject *self, PyObject *args)
+{
+    int temp;
+    if (!PyArg_ParseTuple(args, "i", &temp))
+        return NULL;
+    if(temp <= 0){
+        PyErr_SetString(PyExc_ValueError, "max_fd value out of range ");
+        return NULL;
+    }
+    max_fd = temp;
+    Py_RETURN_NONE;
+}
+
+PyObject *
+meinheld_get_picoev_max_fd(PyObject *self, PyObject *args)
+{
+    return Py_BuildValue("i", max_fd);
+}
 
 PyObject *
 meinheld_set_max_content_length(PyObject *self, PyObject *args)
 {
-    if (!PyArg_ParseTuple(args, "i", &max_content_length))
+    int temp;
+    if (!PyArg_ParseTuple(args, "i", &temp))
         return NULL;
+    if(temp <= 0){
+        PyErr_SetString(PyExc_ValueError, "max_content_length value out of range ");
+        return NULL;
+    }
+    max_content_length = temp;
     Py_RETURN_NONE;
 }
 
@@ -1084,12 +1145,18 @@ static PyMethodDef WsMethods[] = {
     {"access_log", meinheld_access_log, METH_VARARGS, "set access log file path."},
     {"error_log", meinheld_error_log, METH_VARARGS, "set error log file path."},
 
-    {"set_keepalive", meinheld_set_keepalive, METH_VARARGS, "set keep-alive support. default 0. (disable keep-alive)"},
+    {"set_keepalive", meinheld_set_keepalive, METH_VARARGS, "set keep-alive support. value set timeout sec. default 0. (disable keep-alive)"},
     {"get_keepalive", meinheld_get_keepalive, METH_VARARGS, "return keep-alive support."},
     
     {"set_max_content_length", meinheld_set_max_content_length, METH_VARARGS, "set max_content_length"},
     {"get_max_content_length", meinheld_get_max_content_length, METH_VARARGS, "return max_content_length"},
     
+    {"set_backlog", meinheld_set_backlog, METH_VARARGS, "set backlog size"},
+    {"get_backlog", meinheld_get_backlog, METH_VARARGS, "return backlog size"},
+
+    {"set_picoev_max_fd", meinheld_set_picoev_max_fd, METH_VARARGS, "set picoev max fd size"},
+    {"get_picoev_max_fd", meinheld_get_picoev_max_fd, METH_VARARGS, "return picoev max fd size"},
+
     {"set_process_name", meinheld_set_process_name, METH_VARARGS, "set process name"},
     {"stop", meinheld_stop, METH_VARARGS, "stop main loop"},
     // support gunicorn 
