@@ -117,7 +117,10 @@ inner_write(NSocketObject *socket)
     printf("nsocket write fd:%d bytes:%d \n", socket->fd, r);
 #endif
     switch (r) {
-       case -1:
+        case -1:
+#ifdef DEBUG
+            printf("inner_write fd:%d err:%d \n", socket->fd, errno);
+#endif
             if (errno == EAGAIN || errno == EWOULDBLOCK) {
                 return NULL;
             }else{
@@ -126,6 +129,9 @@ inner_write(NSocketObject *socket)
                 return NULL;
             }
         default:
+#ifdef DEBUG
+            printf("inner_write fd:%d read:%d \n", socket->fd, r);
+#endif
             //all done
             //switch 
             free_buffer(socket->send_buf);
@@ -145,7 +151,10 @@ inner_write_all(NSocketObject *socket)
     printf("nsocket write fd:%d bytes:%d \n", socket->fd, r);
 #endif
     switch (r) {
-       case -1:
+        case -1:
+#ifdef DEBUG
+            printf("inner_write_all fd:%d err:%d \n", socket->fd, errno);
+#endif
             if (errno == EAGAIN || errno == EWOULDBLOCK) {
                 return NULL;
             }else{
@@ -154,6 +163,9 @@ inner_write_all(NSocketObject *socket)
                 return NULL;
             }
         default:
+#ifdef DEBUG
+            printf("inner_write_all fd:%d read:%d \n", socket->fd, r);
+#endif
             send_buf->len -=r;
             if(!send_buf->len){
                 //all done
@@ -178,6 +190,9 @@ inner_read(NSocketObject *socket)
     //picoev_set_timeout(loop, socket->fd, 5);
     switch (r) {
         case -1:
+#ifdef DEBUG
+            printf("inner_read fd:%d err:%d \n", socket->fd, errno);
+#endif
             if (errno == EAGAIN || errno == EWOULDBLOCK) {
                 return NULL;
             } else {
@@ -186,12 +201,12 @@ inner_read(NSocketObject *socket)
                 return NULL;
             }
         default:
+#ifdef DEBUG
+            printf("inner_read fd:%d read:%d \n", socket->fd, r);
+#endif
             write2buf(recv_buf, buf, r);
             //all done
             //switch 
-#ifdef DEBUG
-            printf("recv_inner fd:%d \n", socket->fd);
-#endif
             return getPyString(socket->recv_buf);
     }
 }
@@ -212,6 +227,9 @@ inner_recv(NSocketObject *socket, ssize_t len)
 
         if(PyErr_Occurred()){
             // check socket Error
+#ifdef DEBUG
+            printf("inner_recv error occured fd:%d \n", socket->fd);
+#endif
             return NULL;
         }
     
@@ -219,6 +237,9 @@ inner_recv(NSocketObject *socket, ssize_t len)
 
         if(PyErr_Occurred()){
             // check time out
+#ifdef DEBUG
+            printf("inner_recv timeout error occured fd:%d \n", socket->fd);
+#endif
             return NULL;
         }
     }
@@ -245,6 +266,9 @@ inner_send(NSocketObject *socket, char *buf, ssize_t len)
 
         if(PyErr_Occurred()){
             // check socket Error
+#ifdef DEBUG
+            printf("inner_send error occured fd:%d \n", socket->fd);
+#endif
             return NULL;
         }
     
@@ -252,6 +276,9 @@ inner_send(NSocketObject *socket, char *buf, ssize_t len)
 
         if(PyErr_Occurred()){
             // check time out
+#ifdef DEBUG
+            printf("inner_send error occured fd:%d \n", socket->fd);
+#endif
             return NULL;
         }
     }
@@ -276,6 +303,9 @@ inner_sendall(NSocketObject *socket, char *buf, ssize_t len)
 
         if(PyErr_Occurred()){
             // check socket Error
+#ifdef DEBUG
+            printf("inner_sendall error occured fd:%d \n", socket->fd);
+#endif
             return NULL;
         }
     
@@ -283,6 +313,9 @@ inner_sendall(NSocketObject *socket, char *buf, ssize_t len)
 
         if(PyErr_Occurred()){
             // check time out
+#ifdef DEBUG
+            printf("inner_sendall error occured fd:%d \n", socket->fd);
+#endif
             return NULL;
         }
     }
